Package: conos
Title: Clustering on Network of Samples
Version: 1.5.4
Authors@R: c(person("Viktor","Petukhov", email="viktor.s.petuhov@ya.ru", role="aut"), person("Nikolas","Barkas", email="nikolas_barkas@hms.harvard.edu", role="aut"), person("Peter", "Kharchenko", email = "peter_kharchenko@hms.harvard.edu", role = "aut"), person("Weiliang", "Qiu", email = "weiliang.qiu@gmail.com", role = c("ctb")), person("Evan", "Biederstedt", email="evan.biederstedt@gmail.com", role=c("aut", "cre")))
Description: Wires together large collections of single-cell RNA-seq datasets, which allows for both the identification of recurrent cell clusters and the propagation of information between datasets in multi-sample or atlas-scale collections. 'Conos' focuses on the uniform mapping of homologous cell types across heterogeneous sample collections. For instance, users could investigate a collection of dozens of peripheral blood samples from cancer patients combined with dozens of controls, which perhaps includes samples of a related tissue such as lymph nodes. This package interacts with data available through the 'conosPanel' package, which is available in a 'drat' repository. To access this data package, see the instructions at <https://github.com/kharchenkolab/conos>. The size of the 'conosPanel' package is approximately 12 MB.
License: GPL-3
Copyright: See the file COPYRIGHTS for various conos copyright details
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0), Matrix, igraph
biocViews:
Imports: abind, cowplot, ComplexHeatmap, dendextend, dplyr, ggplot2,
        ggrepel, gridExtra, irlba, leidenAlg, magrittr, methods, N2R,
        pagoda2, parallel, R6, reshape2, rlang, Rtsne, sccore (>=
        1.0.0), stats, tools, utils
RoxygenNote: 7.2.3
Suggests: AnnotationDbi, BiocParallel, conosPanel, drat, DESeq2,
        entropy, ggrastr, GO.db, jsonlite, knitr, org.Hs.eg.db,
        org.Mm.eg.db, p2data, PMA, plyr, rhdf5, rmarkdown, rmumps,
        Seurat, shinycssloaders, SummarizedExperiment, testthat,
        tibble, uwot, zoo
Additional_repositories: https://kharchenkolab.github.io/drat/
URL: https://github.com/kharchenkolab/conos
BugReports: https://github.com/kharchenkolab/conos/issues
NeedsCompilation: yes
LinkingTo: Rcpp, RcppArmadillo, RcppEigen, RcppProgress
Author: Viktor Petukhov [aut],
  Nikolas Barkas [aut],
  Peter Kharchenko [aut],
  Weiliang Qiu [ctb],
  Evan Biederstedt [aut, cre]
Maintainer: Evan Biederstedt <evan.biederstedt@gmail.com>
Packaged: 2026-04-25 07:20:07 UTC; evanbiederstedt
Repository: CRAN
Date/Publication: 2026-04-25 07:40:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-25 08:19:47 UTC; unix
Archs: conos.so.dSYM
