# Generated by roxygen2: do not edit by hand

S3method(buildWijMatrix,CsparseMatrix)
S3method(buildWijMatrix,TsparseMatrix)
export("edgeMat<-")
export(Conos)
export(basicSeuratProc)
export(bestClusterThresholds)
export(bestClusterTreeThresholds)
export(buildWijMatrix)
export(convertToPagoda2)
export(edgeMat)
export(embeddingPlot)
export(estimateWeightEntropyPerCell)
export(findSubcommunities)
export(getBetweenCellTypeCorrectedDE)
export(getBetweenCellTypeDE)
export(getCellNames)
export(getClustering)
export(getCountMatrix)
export(getEmbedding)
export(getGeneExpression)
export(getGenes)
export(getOverdispersedGenes)
export(getPca)
export(getPerCellTypeDE)
export(getRawCountMatrix)
export(getSampleNamePerCell)
export(greedyModularityCut)
export(leiden.community)
export(p2app4conos)
export(plotClusterBarplots)
export(plotClusterBoxPlotsByAppType)
export(plotComponentVariance)
export(plotDEheatmap)
export(projectKNNs)
export(rawMatricesWithCommonGenes)
export(saveConosForScanPy)
export(saveDEasCSV)
export(saveDEasJSON)
export(scanKModularity)
export(sgdBatches)
export(stableTreeClusters)
export(velocityInfoConos)
import(Matrix)
import(R6)
import(Rtsne)
import(ggrepel)
import(gridExtra)
import(igraph)
import(leidenAlg)
import(methods)
import(parallel)
import(sccore)
importFrom(ComplexHeatmap,Heatmap)
importFrom(ComplexHeatmap,HeatmapAnnotation)
importFrom(ComplexHeatmap,ht_opt)
importFrom(dplyr,"%>%")
importFrom(grDevices,adjustcolor)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.size)
importFrom(grDevices,rainbow)
importFrom(graphics,par)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,induced.subgraph)
importFrom(igraph,membership)
importFrom(igraph,walktrap.community)
importFrom(magrittr,"%$%")
importFrom(magrittr,"%<>%")
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,as.hclust)
importFrom(stats,cor)
importFrom(stats,dendrapply)
importFrom(stats,dnorm)
importFrom(stats,hclust)
importFrom(stats,is.leaf)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,order.dendrogram)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,relevel)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,packageVersion)
importFrom(utils,write.table)
useDynLib(conos)
