# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

RjnmfC <- function(Xs, Xu, k, alpha, lambda, epsilon, maxiter, verbose) {
    .Call('_conos_RjnmfC', PACKAGE = 'conos', Xs, Xu, k, alpha, lambda, epsilon, maxiter, verbose)
}

adjustedRandcpp <- function(cl1, cl1u, cl2, cl2u, mm1, mm2, nn, fflag) {
    .Call('_conos_adjustedRandcpp', PACKAGE = 'conos', cl1, cl1u, cl2, cl2u, mm1, mm2, nn, fflag)
}

checkBits <- function() {
    .Call('_conos_checkBits', PACKAGE = 'conos')
}

checkOpenMP <- function() {
    .Call('_conos_checkOpenMP', PACKAGE = 'conos')
}

cpcaF <- function(cov, ng, ncomp = 10L, maxit = 1000L, tol = 1e-6, eigenvR = NULL, verbose = TRUE) {
    .Call('_conos_cpcaF', PACKAGE = 'conos', cov, ng, ncomp, maxit, tol, eigenvR, verbose)
}

greedyModularityCutC <- function(merges, deltaM, N, minsize, labels, minbreadth, flatCut) {
    .Call('_conos_greedyModularityCutC', PACKAGE = 'conos', merges, deltaM, N, minsize, labels, minbreadth, flatCut)
}

treeJaccard <- function(merges, clusters, clusterTotals, clmerges = NULL) {
    .Call('_conos_treeJaccard', PACKAGE = 'conos', merges, clusters, clusterTotals, clmerges)
}

scoreTreeConsistency <- function(test, ref, leafidmap, minsize = 10L) {
    .Call('_conos_scoreTreeConsistency', PACKAGE = 'conos', test, ref, leafidmap, minsize)
}

maxStableClusters <- function(merges, thresholds, minthreshold = 0.8, minsize = 10L) {
    .Call('_conos_maxStableClusters', PACKAGE = 'conos', merges, thresholds, minthreshold, minsize)
}

pareDownHubEdges <- function(sY, rowN, k, klow = -1L) {
    .Call('_conos_pareDownHubEdges', PACKAGE = 'conos', sY, rowN, k, klow)
}

getSumWeightMatrix <- function(weights, row_inds, col_inds, factor_levels, normalize = TRUE) {
    .Call('_conos_getSumWeightMatrix', PACKAGE = 'conos', weights, row_inds, col_inds, factor_levels, normalize)
}

adjustWeightsByCellBalancingC <- function(weights, row_inds, col_inds, factor_levels, dividers) {
    .Call('_conos_adjustWeightsByCellBalancingC', PACKAGE = 'conos', weights, row_inds, col_inds, factor_levels, dividers)
}

referenceWij <- function(i, j, d, threads, perplexity) {
    .Call('_conos_referenceWij', PACKAGE = 'conos', i, j, d, threads, perplexity)
}

as_factor <- function(vals) {
    .Call('_conos_as_factor', PACKAGE = 'conos', vals)
}

get_nearest_neighbors <- function(adjacency_list, transition_probabilities, n_verts = 0L, n_cores = 1L, min_prob = 1e-3, min_visited_verts = 1000L, min_prob_lower = 1e-5, max_hitting_nn_num = 0L, max_commute_nn_num = 0L, verbose = TRUE) {
    .Call('_conos_get_nearest_neighbors', PACKAGE = 'conos', adjacency_list, transition_probabilities, n_verts, n_cores, min_prob, min_visited_verts, min_prob_lower, max_hitting_nn_num, max_commute_nn_num, verbose)
}

sgd <- function(coords, targets_i, sources_j, ps, weights, gamma, rho, n_samples, M, alpha, momentum, useDegree, seed, threads, verbose) {
    .Call('_conos_sgd', PACKAGE = 'conos', coords, targets_i, sources_j, ps, weights, gamma, rho, n_samples, M, alpha, momentum, useDegree, seed, threads, verbose)
}

propagate_labels <- function(edge_verts, edge_weights, vert_labels, max_n_iters = 10L, verbose = TRUE, diffusion_fading = 10, diffusion_fading_const = 0.5, tol = 5e-3, fixed_initial_labels = FALSE) {
    .Call('_conos_propagate_labels', PACKAGE = 'conos', edge_verts, edge_weights, vert_labels, max_n_iters, verbose, diffusion_fading, diffusion_fading_const, tol, fixed_initial_labels)
}

smooth_count_matrix <- function(edge_verts, edge_weights, count_matrix, is_label_fixed, max_n_iters = 10L, diffusion_fading = 1.0, diffusion_fading_const = 0.1, tol = 1e-3, verbose = TRUE, normalize = FALSE) {
    .Call('_conos_smooth_count_matrix', PACKAGE = 'conos', edge_verts, edge_weights, count_matrix, is_label_fixed, max_n_iters, diffusion_fading, diffusion_fading_const, tol, verbose, normalize)
}

adjacent_vertices <- function(edge_verts) {
    .Call('_conos_adjacent_vertices', PACKAGE = 'conos', edge_verts)
}

adjacent_vertex_weights <- function(edge_verts, edge_weights) {
    .Call('_conos_adjacent_vertex_weights', PACKAGE = 'conos', edge_verts, edge_weights)
}

spcov <- function(m, cm) {
    .Call('_conos_spcov', PACKAGE = 'conos', m, cm)
}

arma_mat_cor <- function(m) {
    .Call('_conos_arma_mat_cor', PACKAGE = 'conos', m)
}

