% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{getGlobalClusterMarkers}
\alias{getGlobalClusterMarkers}
\title{Deprecated; Get markers for global clusters}
\usage{
getGlobalClusterMarkers(
  conos.obj,
  clustering = "multi level",
  min.samples.expressing = 0,
  min.percent.samples.expressing = 0
)
}
\arguments{
\item{conos.obj}{conos object}

\item{clustering}{character Name of the clustering to use (default='multi level')}

\item{min.samples.expressing}{numeric Minimum number of samples that must have the genes upregulated in the respective cluster (default=0)}

\item{min.percent.samples.expressing}{numeric Minumum percent of samples that must have the gene upregulated (default=0)}
}
\value{
List of consistent genes per each cluster
}
\description{
Deprecated; Get markers for global clusters
}
\keyword{internal}
