% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conos.R
\name{getPercentGlobalClusters}
\alias{getPercentGlobalClusters}
\title{Evaluate how many clusters are global}
\usage{
getPercentGlobalClusters(
  p2list,
  pjc,
  pc.samples.cutoff = 0.9,
  min.cell.count.per.sample = 10
)
}
\arguments{
\item{p2list}{list of pagoda2 object on which clustering was generated}

\item{pjc}{the result of joint clustering}

\item{pc.samples.cutoff}{numeric The percent of the number of the total samples that a cluster has to span to be considered global (default=0.9)}

\item{min.cell.count.per.sample}{numeric The minimum number of cells of cluster in sample to be considered as represented in that sample (default=10)}
}
\value{
percent of clusters that are global given the above criteria
}
\description{
Evaluate how many clusters are global
}
\keyword{internal}
