% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/access_wrappers.R
\name{edgeMat<-}
\alias{edgeMat<-}
\alias{edgeMat<-,Pagoda2-method}
\alias{edgeMat<-,seurat-method}
\alias{edgeMat<-,Seurat-method}
\alias{edgeMat}
\alias{edgeMat,Pagoda2-method}
\alias{edgeMat,seurat-method}
\alias{edgeMat,Seurat-method}
\title{Set edge matrix edgeMat with certain values on sample}
\usage{
edgeMat(sample) <- value

\S4method{edgeMat}{Pagoda2}(sample) <- value

\S4method{edgeMat}{seurat}(sample) <- value

\S4method{edgeMat}{Seurat}(sample) <- value

edgeMat(sample)

\S4method{edgeMat}{Pagoda2}(sample)

\S4method{edgeMat}{seurat}(sample)

\S4method{edgeMat}{Seurat}(sample)
}
\arguments{
\item{sample}{sample from which to access edge matrix edgeMat}

\item{value}{values to set with edgeMat<-}
}
\description{
Set edge matrix edgeMat with certain values on sample

Access edgeMat from sample
}
