% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conos.R
\name{propagateLabelsDiffusion}
\alias{propagateLabelsDiffusion}
\title{Estimate labeling distribution for each vertex, based on provided labels using Random Walk}
\usage{
propagateLabelsDiffusion(
  graph,
  labels,
  max.iters = 100,
  diffusion.fading = 10,
  diffusion.fading.const = 0.1,
  tol = 0.025,
  verbose = TRUE,
  fixed.initial.labels = TRUE
)
}
\arguments{
\item{graph}{input graph}

\item{labels}{vector of factor or character labels, named by cell names}

\item{max.iters}{maximal number of iterations (default=100)}

\item{tol}{numeric Absolute tolerance as a stopping criteria (default=0.025)}

\item{verbose}{boolean Verbose mode (default=TRUE)}

\item{fixed.initial.labels}{boolean Prohibit changes of initial labels during diffusion (default=TRUE)}
}
\description{
Estimate labeling distribution for each vertex, based on provided labels using Random Walk
}
\keyword{internal}
