% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/access_wrappers.R
\name{getGenes}
\alias{getGenes}
\alias{getGenes,Pagoda2-method}
\alias{getGenes,seurat-method}
\alias{getGenes,Seurat-method}
\alias{getGenes,Conos-method}
\title{Access genes from sample}
\usage{
getGenes(sample)

\S4method{getGenes}{Pagoda2}(sample)

\S4method{getGenes}{seurat}(sample)

\S4method{getGenes}{Seurat}(sample)

\S4method{getGenes}{Conos}(sample)
}
\arguments{
\item{sample}{sample from which to get genes}
}
\description{
Access genes from sample
}
