% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/access_wrappers.R
\name{getClustering}
\alias{getClustering}
\alias{getClustering,Pagoda2-method}
\alias{getClustering,seurat-method}
\alias{getClustering,Seurat-method}
\alias{getClustering,Conos-method}
\title{Access clustering from sample}
\usage{
getClustering(sample, type)

\S4method{getClustering}{Pagoda2}(sample, type)

\S4method{getClustering}{seurat}(sample, type)

\S4method{getClustering}{Seurat}(sample, type)

\S4method{getClustering}{Conos}(sample, type)
}
\arguments{
\item{sample}{sample from which to get the clustering}

\item{type}{character Type of clustering to get}
}
\description{
Access clustering from sample
}
