% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/access_wrappers.R
\name{getEmbedding}
\alias{getEmbedding}
\alias{getEmbedding,Pagoda2-method}
\alias{getEmbedding,seurat-method}
\alias{getEmbedding,Seurat-method}
\alias{getEmbedding,Conos-method}
\title{Access embedding from sample}
\usage{
getEmbedding(sample, type)

\S4method{getEmbedding}{Pagoda2}(sample, type)

\S4method{getEmbedding}{seurat}(sample, type)

\S4method{getEmbedding}{Seurat}(sample, type)

\S4method{getEmbedding}{Conos}(sample, type)
}
\arguments{
\item{sample}{sample from which to get the embedding}

\item{type}{character Type of embedding to get}
}
\description{
Access embedding from sample
}
