% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/de_functions.R
\name{saveDEasJSON}
\alias{saveDEasJSON}
\title{Save differential expression results as JSON}
\usage{
saveDEasJSON(
  de.results = NULL,
  saveprefix = NULL,
  gene.metadata = NULL,
  cluster.sep.chr = "<!!>"
)
}
\arguments{
\item{de.results}{differential expression results (default=NULL)}

\item{saveprefix}{prefix for the differential expression output (default=NULL)}

\item{gene.metadata}{data.frame with gene metadata (default=NULL)}

\item{cluster.sep.chr}{character string of length 1 specifying a delimiter to separate cluster and app names (default='<!!>')}
}
\value{
JSON with DE results
}
\description{
Save differential expression results as JSON
}
