% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{namedLevels}
\alias{namedLevels}
\title{Get a vector with the levels of a factor named with their
own name. Useful for lapply loops over factor levels}
\usage{
namedLevels(f)
}
\arguments{
\item{f}{a factor}
}
\value{
named character vector of factor levels
}
\description{
Get a vector with the levels of a factor named with their
own name. Useful for lapply loops over factor levels
}
\keyword{internal}
