% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrations.R
\name{armaCor}
\alias{armaCor}
\title{A slightly faster way of calculating column correlation matrix}
\usage{
armaCor(mat, nthreads = 1)
}
\arguments{
\item{mat}{matrix whose columns will be correlated}

\item{nthreads}{number of threads to use}
}
\value{
correlation matrix
}
\description{
A slightly faster way of calculating column correlation matrix
}
\keyword{internal}
