% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/access_wrappers.R
\name{getPca}
\alias{getPca}
\alias{getPca,Pagoda2-method}
\alias{getPca,seurat-method}
\alias{getPca,Seurat-method}
\title{Access PCA from sample}
\usage{
getPca(sample)

\S4method{getPca}{Pagoda2}(sample)

\S4method{getPca}{seurat}(sample)

\S4method{getPca}{Seurat}(sample)
}
\arguments{
\item{sample}{sample from which to access PCA}
}
\description{
Access PCA from sample
}
