% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conos.R
\name{getOdGenesUniformly}
\alias{getOdGenesUniformly}
\title{Get top overdispersed genes across samples}
\usage{
getOdGenesUniformly(samples, n.genes)
}
\arguments{
\item{samples}{list of pagoda2 objects}

\item{n.genes}{number of overdispersed genes to extract}
}
\description{
Get top overdispersed genes across samples
}
\keyword{internal}
