% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/de_functions.R
\name{saveDEasCSV}
\alias{saveDEasCSV}
\title{Save differential expression as table in *csv format}
\usage{
saveDEasCSV(de.results, saveprefix, gene.metadata = NULL)
}
\arguments{
\item{de.results}{output of differential expression results, corrected or uncorrected}

\item{saveprefix}{character prefix for output file}

\item{gene.metadata}{gene metadta to include (default=NULL)}
}
\description{
Save differential expression as table in *csv format
}
