% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conos.R
\name{findSubcommunities}
\alias{findSubcommunities}
\title{Increase resolution for a specific set of clusters}
\usage{
findSubcommunities(
  con,
  target.clusters,
  clustering = NULL,
  groups = NULL,
  method = leiden.community,
  ...
)
}
\arguments{
\item{con}{conos object}

\item{target.clusters}{clusters for which the resolution should be increased}

\item{clustering}{name of clustering in the conos object to use. Either 'clustering' or 'groups' must be provided (default=NULL).}

\item{groups}{set of clusters to use. Ignored if 'clustering' is not NULL (default=NULL).}

\item{method}{function, used to find communities (default=leiden.community).}

\item{...}{additional params passed to the community function}
}
\value{
set of clusters with increased resolution
}
\description{
Increase resolution for a specific set of clusters
}
