% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrations.R
\name{basicSeuratProc}
\alias{basicSeuratProc}
\title{Create and preprocess a Seurat object}
\usage{
basicSeuratProc(
  count.matrix,
  vars.to.regress = NULL,
  verbose = TRUE,
  do.par = TRUE,
  n.pcs = 100,
  cluster = TRUE,
  tsne = TRUE,
  umap = FALSE
)
}
\arguments{
\item{count.matrix}{gene count matrix}

\item{vars.to.regress}{variables to regress with Seurat (default=NULL)}

\item{verbose}{boolean Verbose mode (default=TRUE)}

\item{do.par}{boolean Use parallel processing for regressing out variables faster (default=TRUE)}

\item{n.pcs}{numeric Number of principal components (default=100)}

\item{cluster}{boolean Whether to perform clustering (default=TRUE)}

\item{tsne}{boolean Whether to construct tSNE embedding (default=TRUE)}

\item{umap}{boolean Whether to construct UMAP embedding, works only for Seurat v2.3.1 or higher (default=FALSE)}
}
\value{
Seurat object
}
\description{
Create and preprocess a Seurat object
}
