% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conos.R
\name{quickPlainPCA}
\alias{quickPlainPCA}
\title{Use space of combined sample-specific PCAs as a space}
\usage{
quickPlainPCA(
  r.n,
  data.type = "counts",
  ncomps = 30,
  n.odgenes = NULL,
  var.scale = TRUE,
  verbose = TRUE,
  score.component.variance = FALSE,
  n.cores = 1
)
}
\arguments{
\item{r.n}{list of pagoda2 objects}

\item{data.type}{character Type of data type in the input pagoda2 objects within r.n (default='counts')}

\item{ncomps}{numeric Number of components to calculate (default=30)}

\item{n.odgenes}{numeric Number of overdispersed genes to take from each dataset (default=NULL)}

\item{var.scale}{boolean Whether to scale variance (default=TRUE)}

\item{verbose}{boolean Whether to be verbose (default=TRUE)}

\item{score.component.variance}{boolean Whether to score component variance (default=FALSE)}

\item{n.cores}{numeric Number of cores to use (default=1)}
}
\value{
PCA projection, using space of combined sample-specific PCAs
}
\description{
Use space of combined sample-specific PCAs as a space
}
\keyword{internal}
