% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plotClusterBarplots}
\alias{plotClusterBarplots}
\title{Plots barplots per sample of composition of each pagoda2 application based on
selected clustering}
\usage{
plotClusterBarplots(
  conos.obj = NULL,
  clustering = NULL,
  groups = NULL,
  sample.factor = NULL,
  show.entropy = TRUE,
  show.size = TRUE,
  show.composition = TRUE,
  legend.height = 0.2
)
}
\arguments{
\item{conos.obj}{A conos object (default=NULL)}

\item{clustering}{name of clustering in the current object (default=NULL)}

\item{groups}{arbitrary grouping of cells (to use instead of the clustering) (default=NULL)}

\item{sample.factor}{a factor describing cell membership in the samples (or some other category) (default=NULL). This will default to samples if not provided.}

\item{show.entropy}{boolean Whether to include entropy barplot (default=TRUE)}

\item{show.size}{boolean Whether to include size barplot (default=TRUE)}

\item{show.composition}{boolean Whether to include composition barplot (default=TRUE)}

\item{legend.height}{numeric Relative hight of the legend panel (default=0.2)}
}
\value{
a ggplot object
}
\description{
Plots barplots per sample of composition of each pagoda2 application based on
selected clustering
}
