% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conos.R
\name{estimateWeightEntropyPerCell}
\alias{estimateWeightEntropyPerCell}
\title{Estimate entropy of edge weights per cell according to the specified factor.
Can be used to visualize alignment quality according to this factor.}
\usage{
estimateWeightEntropyPerCell(con, factor.per.cell)
}
\arguments{
\item{con}{conos object}

\item{factor.per.cell}{some factor, which group cells, such as sample or a specific condition}
}
\value{
entropy of edge weights per cell
}
\description{
Estimate entropy of edge weights per cell according to the specified factor.
Can be used to visualize alignment quality according to this factor.
}
