% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{small_panel.preprocessed}
\alias{small_panel.preprocessed}
\title{Small pre-processed data from Pagoda2, two samples, each dimension (1000, 100)}
\format{
An object of class \code{list} of length 2.
}
\usage{
small_panel.preprocessed
}
\description{
Small pre-processed data from Pagoda2, two samples, each dimension (1000, 100)
}
\keyword{datasets}
