% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conos.R
\name{getSampleNamePerCell}
\alias{getSampleNamePerCell}
\title{Retrieve sample names per cell}
\usage{
getSampleNamePerCell(samples)
}
\arguments{
\item{samples}{list of samples}
}
\value{
list of sample names
getSampleNamePerCell(small_panel.preprocessed)
}
\description{
Retrieve sample names per cell
}
