% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{getClusteringGroups}
\alias{getClusteringGroups}
\title{Extract specified clustering from list of conos clusterings}
\usage{
getClusteringGroups(clusters, clustering)
}
\arguments{
\item{clusters}{list of conos clusterings}

\item{clustering}{name of extracted clustering}
}
\value{
vector of clusters, named with cell names
}
\description{
Extract specified clustering from list of conos clusterings
}
\keyword{internal}
