% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/access_wrappers.R
\name{getRawCountMatrix}
\alias{getRawCountMatrix}
\alias{getRawCountMatrix,Pagoda2-method}
\alias{getRawCountMatrix,seurat-method}
\alias{getRawCountMatrix,Seurat-method}
\alias{getRawCountMatrix,Conos-method}
\title{Access raw count matrix from sample}
\usage{
getRawCountMatrix(sample, transposed = FALSE)

\S4method{getRawCountMatrix}{Pagoda2}(sample, transposed = FALSE)

\S4method{getRawCountMatrix}{seurat}(sample, transposed = FALSE)

\S4method{getRawCountMatrix}{Seurat}(sample, transposed = FALSE)

\S4method{getRawCountMatrix}{Conos}(sample, transposed = FALSE)
}
\arguments{
\item{sample}{sample from which to get the raw count matrix}

\item{transposed}{boolean Whether the raw count matrix should be transposed (default=FALSE)}
}
\description{
Access raw count matrix from sample
}
