% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/de_functions.R
\name{checkCountsWholeNumbers}
\alias{checkCountsWholeNumbers}
\title{Check that the count data contain only integer counts}
\usage{
checkCountsWholeNumbers(input.matrix)
}
\arguments{
\item{input.matrix}{the count data from aggregated samples input to DESeq}
}
\value{
if non-integer counts are found, an error is returned
}
\description{
Check that the count data contain only integer counts
}
\keyword{internal}
