% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/access_wrappers.R
\name{getCellNames}
\alias{getCellNames}
\alias{getCellNames,Pagoda2-method}
\alias{getCellNames,seurat-method}
\alias{getCellNames,Seurat-method}
\alias{getCellNames,Conos-method}
\title{Access cell names from sample}
\usage{
getCellNames(sample)

\S4method{getCellNames}{Pagoda2}(sample)

\S4method{getCellNames}{seurat}(sample)

\S4method{getCellNames}{Seurat}(sample)

\S4method{getCellNames}{Conos}(sample)
}
\arguments{
\item{sample}{sample from which to cell names}
}
\description{
Access cell names from sample
}
