% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/de_functions.R
\name{rawMatricesWithCommonGenes}
\alias{rawMatricesWithCommonGenes}
\title{Get raw matrices with common genes}
\usage{
rawMatricesWithCommonGenes(con.obj, sample.groups = NULL)
}
\arguments{
\item{con.obj}{Conos object}

\item{sample.groups}{list of samples to select from Conos object, con.obj$samples (default=NULL)}
}
\value{
raw matrices subset with common genes
}
\description{
Get raw matrices with common genes
}
