% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plotClusterBoxPlotsByAppType}
\alias{plotClusterBoxPlotsByAppType}
\title{Generate boxplot per cluster of the proportion of cells in each celltype}
\usage{
plotClusterBoxPlotsByAppType(
  conos.obj,
  clustering = NULL,
  apptypes = NULL,
  return.details = FALSE
)
}
\arguments{
\item{conos.obj}{conos object}

\item{clustering}{name of the clustering to use (default=NULL)}

\item{apptypes}{a factor specifying how to group the samples (default=NULL)}

\item{return.details}{boolean If TRUE return a list with the plot and the summary data.frame (default=FALSE)}
}
\value{
Boxplot per cluster of the proportion of cells in each celltype
}
\description{
Generate boxplot per cluster of the proportion of cells in each celltype
}
