% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conclass.R, R/conos.R, R/plot.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{leiden.community}
\alias{embeddingPlot}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{leidenAlg}{\code{\link[leidenAlg]{leiden.community}}, \code{\link[leidenAlg]{leiden.community}}}

  \item{sccore}{\code{\link[sccore]{embeddingPlot}}}
}}

