% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/access_wrappers.R
\name{getOverdispersedGenes}
\alias{getOverdispersedGenes}
\alias{getOverdispersedGenes,Pagoda2-method}
\alias{getOverdispersedGenes,seurat-method}
\alias{getOverdispersedGenes,Seurat-method}
\alias{getOverdispersedGenes,Conos-method}
\title{Access overdispersed genes from sample}
\usage{
getOverdispersedGenes(sample, n.odgenes = 1000)

\S4method{getOverdispersedGenes}{Pagoda2}(sample, n.odgenes = NULL)

\S4method{getOverdispersedGenes}{seurat}(sample, n.odgenes = NULL)

\S4method{getOverdispersedGenes}{Seurat}(sample, n.odgenes = NULL)

\S4method{getOverdispersedGenes}{Conos}(sample, n.odgenes = NULL)
}
\arguments{
\item{sample}{sample from which to overdispereed genes}

\item{n.odgenes}{numeric Number of overdisperesed genes to get}
}
\description{
Access overdispersed genes from sample
}
